/*
    This file is part of Posixcpp library.

    Posixcpp library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    Posixcpp library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with Posixcpp library. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "posix_signals.hpp"

#include <cstring>
#include <cassert>

using posix::signals::Signal;

using namespace posix;
using namespace posix::signals;

Signal::Signal()
{
	std::memset(&m_sigInfo, 0, sizeof(m_sigInfo));
}

Signal::Signal( Number_t number )
{
	setSigno(number);
}

Signal::Signal( const siginfo_t & siginfo ) :
m_sigInfo(siginfo)
{
	assert( m_sigInfo.si_signo < SIGRTMIN );
}

Number_t Signal::signo() const
{
	return Number_t(m_sigInfo.si_signo);
}

void Signal::setSigno( Number_t value )
{
	m_sigInfo.si_signo = int(value);
}

namespace posix {
namespace signals {

std::ostream & operator <<( std::ostream & stream, Number_t data ) {
    switch (data) {
    case Number_t::UNKNOWN:
        stream << "UNKNOWN"; break;
    case Number_t::HUP:
        stream << "HUP"; break;
    case Number_t::INT:
        stream << "INT"; break;
    case Number_t::QUIT:
        stream << "QUIT"; break;
    case Number_t::ILL:
        stream << "ILL"; break;
    case Number_t::TRAP:
        stream << "TRAP"; break;
    case Number_t::ABRT:
        stream << "ABRT"; break;
    case Number_t::BUS:
        stream << "BUS"; break;
    case Number_t::FPE:
        stream << "FPE"; break;
    case Number_t::SEGV:
        stream << "SEGV"; break;
    case Number_t::USR1:
        stream << "USR1"; break;
    case Number_t::USR2:
        stream << "USR2"; break;
    case Number_t::TERM:
        stream << "TERM"; break;
    default:
        stream << "UNDEFINED"; break;
    }
    stream << " (" << strsignal(int(data)) << ")";

    return stream;
}

std::ostream & operator <<( std::ostream & stream, const Signal & entity ) {
    stream << "Signal:\n\tNumber: " << Number_t(entity.m_sigInfo.si_signo) <<
                       "\tErrno:  " << strerror(entity.m_sigInfo.si_errno) << '\n';

    return stream;
}

}
}
